/***********************************************************
 * $Id: fancysaneoptiondescriptor.h 1667 2015-05-13 22:14:12Z fancylibs $
 *
 * FancyLibs sane wrapper
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSANEOPTIONDESCRIPTOR_H_
#define FANCYSANEOPTIONDESCRIPTOR_H_

#include <fancysane.h>
#include <fancyserializable.h>
#include <fancystring.h>

namespace Fancy {

class List;
class String;

namespace Sane {

class SaneRange;

class FANCYSANE_API SaneOptionDescriptor : virtual public Fancy::Serializable
{

public:
    typedef Fancy::ObjPtr<SaneOptionDescriptor> ptr_t;

    /**
     * Enum of all possible types of options
     */
    enum OptionType {
        OptionTypeBool,
        OptionTypeInt,
        OptionTypeFixed,
        OptionTypeString,
        OptionTypeButton,
        OptionTypeGroup
    };

    /**
     * @return a readable char* of the given OptionType
     */
    static const char *formatOptionType(OptionType optionType);

    /**
     * Enum of all possible units of the options
     */
    enum OptionUnit {
            OptionUnitNone,
            OptionUnitPixel,
            OptionUnitBit,
            OptionUnitMm,
            OptionUnitDpi,
            OptionUnitPercent,
            OptionUnitMicrosecond
    };

    /**
     * @return a readable char* of the given OptionUnit
     */
    static const char *formatOptionUnit(OptionUnit optionUnit);

    /**
     * Enum of all possible constraint types of the options
     */
    enum ConstraintType {
        /**
         * This option has no constraints.
         */
        ConstraintTypeNone,
        /**
         * This option has a constraint of type @c SaneRange
         */
        ConstraintTypeRange,
        /**
         * This option has a constraint of type @c Fancy::ArrayObject<int32_t>
         */
        ConstraintTypeIntList,
        /**
         * This option has a constraint of type @c Fancy::List,
         * the member of the list are of type Fancy::String.
         */
        ConstraintTypeStringList
    };

    /**
     * @return a readable char* of the given ConstraintType
     */
    static const char *formatConstraintType(ConstraintType constraintType);

    static const int CAPABILITY_SOFT_SELECT  = (1 << 0);
    static const int CAPABILITY_HARD_SELECT  = (1 << 1);
    static const int CAPABILITY_SOFT_DETECT  = (1 << 2);
    static const int CAPABILITY_EMULATED     = (1 << 3);
    static const int CAPABILITY_AUTOMATIC    = (1 << 4);
    static const int CAPABILITY_INACTIVE     = (1 << 5);
    static const int CAPABILITY_ADVANCED     = (1 << 6);


    static ptr_t newInstance();

    // implementation of Fancy::Serializable
    static Fancy::Serializable::ptr_t newSerializableInstance();
    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();
    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;
    void writeFields(const Fancy::ObjPtr<Fancy::Serializer> &serializer) const;
    Fancy::Serializable::ptr_t readFields(const Fancy::ObjPtr<Fancy::Serializer> &serializer, int64_t serialVersionUID);

    int getId() const {
        return this->id;
    }

    void setId(int id) {
        this->id = id;
    }

    int getCapabilities() const {
        return this->capabilities;
    }

    void setCapabilities(int capabilities) {
        this->capabilities = capabilities;
    }

    const Fancy::ObjPtr<Fancy::Object>& getConstraint() const {
        return this->constraint;
    }

    void setConstraint(const Fancy::ObjPtr<Fancy::Object>& constraint) {
        this->constraint = constraint;
    }

    ConstraintType getConstraintType() const {
        return this->constraintType;
    }

    void setConstraintType(ConstraintType constraintType) {
        this->constraintType = constraintType;
    }

    const Fancy::ObjPtr<Fancy::String>& getDesc() const {
        return this->desc;
    }

    void setDesc(const Fancy::ObjPtr<Fancy::String>& desc) {
        this->desc = desc;
    }

    const Fancy::ObjPtr<Fancy::String>& getName() const {
        return this->name;
    }

    void setName(const Fancy::ObjPtr<Fancy::String>& name) {
        this->name = name;
    }

    const Fancy::ObjPtr<Fancy::String>& getTitle() const {
        return this->title;
    }

    void setTitle(const Fancy::ObjPtr<Fancy::String>& title) {
        this->title = title;
    }

    OptionType getType() const {
        return this->type;
    }

    void setType(OptionType type) {
        this->type = type;
    }

    OptionUnit getUnit() const {
        return this->unit;
    }

    void setUnit(OptionUnit unit) {
        this->unit = unit;
    }

    int getSize() const {
        return size;
    }

    void setSize(int size) {
        this->size = size;
    }

    bool isSettable() const {
        return (this->capabilities & CAPABILITY_SOFT_SELECT) != 0;
    }

    bool isActive() const {
        return (this->capabilities & CAPABILITY_INACTIVE) == 0;
    }

    std::string toString() const;

protected:
    SaneOptionDescriptor();
    virtual ~SaneOptionDescriptor();
private:

    int id;
    Fancy::ObjPtr<Fancy::String> name;
    Fancy::ObjPtr<Fancy::String> title;
    Fancy::ObjPtr<Fancy::String> desc;
    int size;
    OptionType type;
    OptionUnit unit;
    ConstraintType constraintType;
    int capabilities;
    Fancy::ObjPtr<Fancy::Object> constraint;

};

}}// end namespaces

#endif
